# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-08-31 17:41
# @Author : 毛鹏
loc = 'await page.cdeadapage.'
cleaned_loc = loc.replace('page.', '').replace('await', '')
print(cleaned_loc)


# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2024-09-07 23:25
# @Author : 毛鹏
import random

data_list = [{'exp': 2, 'loc': 'get_by_role("textbox", name="请输入1密码")'},
             {'exp': 0, 'loc': '//input[@type="password"]'}]
print(random.randint(0, len(data_list) - 1), len(data_list))



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description:
# @Time   : 2023-09-09 23:17
# @Author : 毛鹏

from uiautomator2 import UiObject, UiObjectNotFoundError
from uiautomator2.exceptions import XPathElementNotFoundError
from uiautomator2.xpath import XPathSelector

from mangotools.assertion import MangoAssertion
from ..android._application import AndroidApplication
from ..android._assertion import AndroidAssertion
from ..android._customization import AndroidCustomization
from ..android._element import AndroidElement
from ..android._equipment import AndroidEquipment
from ..android._page import AndroidPage
from ...enums import ElementExpEnum
from ...exceptions import MangoAutomationError
from ...exceptions.error_msg import *

__all__ = [
    'AndroidApplication',
    'AndroidAssertion',
    'AndroidElement',
    'AndroidCustomization',
    'AndroidEquipment',
    'AndroidPage',
    'AndroidDriver',
]

from mangotools.mangos import Mango


class AndroidDriver(AndroidPage,
                    AndroidElement,
                    AndroidEquipment,
                    AndroidCustomization,
                    AndroidApplication):

    def __init__(self, base_data):
        super().__init__(base_data)

    def open_app(self):
        if not self.base_data.is_open_app:
            self.base_data.is_open_app = True
            self.a_press_home()
            self.a_app_stop_all()
            if self.base_data.android and self.base_data.package_name:
                self.a_start_app(self.base_data.package_name)

    def a_action_element(self, name, ope_key, ope_value):
        self.base_data.log.debug(f'操作元素，名称：{name},key:{ope_key},value:{ope_value}')
        try:
            Mango.s_e(self, ope_key, ope_value)
        except ValueError as error:
            self.base_data.log.error(f'安卓自动化失败-1，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0012)
        except UiObjectNotFoundError as error:
            self.base_data.log.error(f'安卓自动化失败-2，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0032, value=(name,))
        except XPathElementNotFoundError as error:
            self.base_data.log.error(f'安卓自动化失败-3，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0050, value=(name,))

    def a_assertion_element(self, name, ope_key, ope_value):
        self.base_data.log.debug(f'断言元素，名称：{name},key:{ope_key},value:{ope_value}')
        is_method = callable(getattr(AndroidAssertion(self.base_data), ope_key, None))
        if is_method and ope_value.get('actual') is None:
            raise MangoAutomationError(*ERROR_MSG_0031, value=(name,))

        try:
            if is_method:
                self.base_data.log.debug(f'开始断言-1，方法：{ope_key}，断言值：{ope_value}')
                return Mango.s_e(AndroidAssertion(self.base_data), ope_key, ope_value)
            else:
                self.base_data.log.debug(f'开始断言-2，方法：{ope_key}，断言值：{ope_value}')
                return MangoAssertion(self.base_data.mysql_connect, self.base_data.test_data) \
                    .ass(ope_key, ope_value.get('actual'), ope_value.get('expect'))
        except AssertionError as error:
            self.base_data.log.error(f'安卓自动化失败-1，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0017, value=error.args)
        except AttributeError as error:
            self.base_data.log.error(f'安卓自动化失败-2，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0030, )
        except ValueError as error:
            self.base_data.log.error(f'安卓自动化失败-3，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0005, )

    def a_find_ele(self, name, _type, exp, loc, sub) -> tuple[UiObject, int, str] | tuple[XPathSelector, int, str]:
        self.base_data.log.debug(
            f'查找元素，名称：{name},_type:{_type},exp:{exp},loc:{loc},sub:{sub}')
        match exp:
            case ElementExpEnum.LOCATOR.value:
                try:
                    if loc[:5] == 'xpath':
                        loc = eval(f"self.android.{loc}")
                    else:
                        loc = eval(f"self.android{loc}")
                except SyntaxError:
                    raise MangoAutomationError(*ERROR_MSG_0022)
            case ElementExpEnum.XPATH.value:
                loc = self.base_data.android.xpath(loc)
            case ElementExpEnum.BOUNDS.value:
                loc = self.base_data.android(text=loc)
            case ElementExpEnum.DESCRIPTION.value:
                loc = self.base_data.android(description=loc)
            case ElementExpEnum.RESOURCE_ID.value:
                loc = self.base_data.android(resourceId=loc)
            case _:
                raise MangoAutomationError(*ERROR_MSG_0020)
        text = None
        try:
            text = self.a_get_text(loc)
        except Exception:
            pass
        if exp == ElementExpEnum.XPATH.value:
            count = len(loc.all())
        else:
            count = loc.count
        return loc, count, text



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-04-04 21:42
# @Author : 毛鹏
import re
import traceback

from playwright.async_api import Locator, Error, TimeoutError

from mangotools.assertion import MangoAssertion
from mangotools.enums import StatusEnum
from mangotools.mangos import Mango
from ....enums import ElementExpEnum, ElementOperationEnum
from ....exceptions import MangoAutomationError
from ....exceptions.error_msg import *
from ....uidrive._base_data import BaseData
from ....uidrive.web.async_web._assertion import AsyncWebAssertion
from ....uidrive.web.async_web._browser import AsyncWebBrowser
from ....uidrive.web.async_web._customization import AsyncWebCustomization
from ....uidrive.web.async_web._element import AsyncWebElement
from ....uidrive.web.async_web._input_device import AsyncWebDeviceInput
from ....uidrive.web.async_web._page import AsyncWebPage

re = re
__all__ = [
    'AsyncWebPage',
    'AsyncWebElement',
    'AsyncWebDeviceInput',
    'AsyncWebCustomization',
    'AsyncWebBrowser',
    'AsyncWebAssertion',
    'AsyncWebDevice',
]


class AsyncWebDevice(AsyncWebBrowser,
                     AsyncWebPage,
                     AsyncWebElement,
                     AsyncWebDeviceInput,
                     AsyncWebCustomization):

    def __init__(self, base_data: BaseData):
        super().__init__(base_data)

    async def open_url(self, is_open: bool = False):
        if not self.base_data.is_open_url or is_open:
            self.base_data.log.debug(f'打开url，is_open_url：{self.base_data.is_open_url},url:{self.base_data.url}')
            await self.w_goto(self.base_data.url)
            self.base_data.is_open_url = True

    async def web_action_element(self, name, ope_key, ope_value, ):
        self.base_data.log.debug(f'操作元素，名称：{name},key:{ope_key},value:{ope_value}')
        try:
            await Mango.a_e(self, ope_key, ope_value)
        except TimeoutError as error:
            self.base_data.log.error(f'WEB自动化操作失败-1，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0011, value=(name,))
        except Error as error:
            self.base_data.log.error(f'WEB自动化操作失败-2，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0032, value=(name,))
        except ValueError as error:
            self.base_data.log.error(f'WEB自动化操作失败-3，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0012)

    async def web_assertion_element(self, name, ope_key, ope_value) -> str:
        self.base_data.log.debug(f'断言元素，名称：{name},key:{ope_key},value:{ope_value}')
        is_method = callable(getattr(AsyncWebAssertion, ope_key, None))
        try:
            if is_method:
                if ope_value.get('actual', None) is None:
                    traceback.print_exc()
                    raise MangoAutomationError(*ERROR_MSG_0031, value=(name,))
                self.base_data.log.debug(f'开始断言-1，方法：{ope_key}，断言值：{ope_value}')
                return await Mango.a_e(AsyncWebAssertion(self.base_data), ope_key, ope_value)
            else:
                self.base_data.log.debug(f'开始断言-2，方法：{ope_key}，断言值：{ope_value}')
                return MangoAssertion(self.base_data.mysql_connect, self.base_data.test_data) \
                    .ass(ope_key, ope_value.get('actual'), ope_value.get('expect'))
        except AssertionError as error:
            self.base_data.log.debug(f'WEB自动化断言失败-1，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0017, value=error.args)
        except AttributeError as error:
            self.base_data.log.error(f'WEB自动化断言失败-2，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0048)
        except ValueError as error:
            self.base_data.log.error(f'WEB自动化断言失败-3，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0005)

        except Error as error:
            self.base_data.log.error(f'WEB自动化断言失败-4，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0052, value=(name,), )

    async def web_find_ele(self, name, _type, exp, loc, sub, is_iframe) \
            -> tuple[Locator, int, str] | tuple[list[Locator], int, str]:
        self.base_data.log.debug(
            f'查找元素，名称：{name},_type:{_type},exp:{exp},loc:{loc},sub:{sub},is_iframe:{is_iframe}')
        if is_iframe != StatusEnum.SUCCESS.value:
            locator: Locator = await self.__find_ele(self.base_data.page, exp, loc)
            try:
                count = await locator.count()
                loc = locator.nth(sub - 1) if sub else locator
                try:
                    text = await self.w_get_text(loc)
                except Exception:
                    text = None
                return loc, count, text
            except Error as error:
                self.base_data.log.debug(
                    f'WEB自动化查找元素失败-1，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
                raise MangoAutomationError(*ERROR_MSG_0041, value=(name, loc))
        else:
            ele_list: list[Locator] = []
            for i in self.base_data.page.frames:
                locator: Locator = await self.__find_ele(i, exp, loc)
                try:
                    count = await locator.count()
                except Error as error:
                    self.base_data.log.debug(
                        f'WEB自动化查找元素失败-2，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
                    raise MangoAutomationError(*ERROR_MSG_0041, )
                if count > 0:
                    for nth in range(0, count):
                        ele_list.append(locator.nth(nth))
                else:
                    raise MangoAutomationError(*ERROR_MSG_0023)
            if not ele_list:
                raise MangoAutomationError(*ERROR_MSG_0023)
            if not ele_list and _type == ElementOperationEnum.OPE:
                raise MangoAutomationError(*ERROR_MSG_0029, value=(name, loc))
            try:
                count = len(ele_list)
                loc = ele_list[sub - 1] if sub else ele_list
                try:
                    text = await self.w_get_text(loc)
                except Exception:
                    text = None
                return loc, count, text
            except IndexError:
                raise MangoAutomationError(*ERROR_MSG_0025, value=(len(ele_list),))

    async def __find_ele(self, page, exp, loc) -> Locator:
        match exp:
            case ElementExpEnum.LOCATOR.value:
                try:
                    return eval(f"await page.{loc}")
                except SyntaxError:
                    try:
                        return eval(f"page.{loc}")
                    except SyntaxError as error:
                        self.base_data.log.error(f'WEB自动化查找元素失败-3，类型：{type(error)}，失败详情：{error}')
                        raise MangoAutomationError(*ERROR_MSG_0022)
                    except NameError as error:
                        self.base_data.log.error(f'WEB自动化查找元素失败-4，类型：{type(error)}，失败详情：{error}')
                        raise MangoAutomationError(*ERROR_MSG_0060)
            case ElementExpEnum.XPATH.value:
                return page.locator(f'xpath={loc}')
            case ElementExpEnum.CSS.value:
                return page.locator(loc)
            case ElementExpEnum.TEXT.value:
                return page.get_by_text(loc, exact=True)
            case ElementExpEnum.PLACEHOLDER.value:
                return page.get_by_placeholder(loc)
            case _:
                raise MangoAutomationError(*ERROR_MSG_0020)



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: # @Time   : 2023/4/6 13:31
# @Author : 毛鹏
import os.path

from uiautomator2 import Direction

from mangotools.decorator import sync_method_callback
from mangotools.models import MethodModel
from ...tools import Meta
from ...uidrive._base_data import BaseData


class AndroidPage(metaclass=Meta):
    """页面操作"""

    def __init__(self, base_data: BaseData):
        self.base_data = base_data

    @sync_method_callback('android', '页面操作', 1)
    def a_swipe_right(self):
        """右滑"""
        self.base_data.android.swipe_ext(Direction.HORIZ_FORWARD)

    @sync_method_callback('android', '页面操作', 2)
    def a_swipe_left(self):
        """左滑"""
        self.base_data.android.swipe_ext(Direction.HORIZ_BACKWARD)

    @sync_method_callback('android', '页面操作', 3)
    def a_swipe_up(self):
        """上滑"""
        self.base_data.android.swipe_ext(Direction.FORWARD)

    @sync_method_callback('android', '页面操作', 4)
    def a_swipe_down(self):
        """下滑"""
        self.base_data.android.swipe_ext(Direction.BACKWARD)

    @sync_method_callback('android', '页面操作', 5, [
        MethodModel(n='当前x', f='sx', p='请输入sx坐标', d=True),
        MethodModel(n='当前y', f='sy', p='请输入sy坐标', d=True),
        MethodModel(n='目标x', f='ex', p='请输入ex坐标', d=True),
        MethodModel(n='目标y', f='ey', p='请输入ey坐标', d=True)])
    def a_swipe(self, sx, sy, ex, ey):
        """坐标滑动"""
        self.base_data.android.swipe(sx, sy, ex, ey, 0.5)

    @sync_method_callback('android', '页面操作', 6, [
        MethodModel(n='当前x', f='sx', p='请输入sx坐标', d=True),
        MethodModel(n='当前y', f='sy', p='请输入sy坐标', d=True),
        MethodModel(n='目标x', f='ex', p='请输入ex坐标', d=True),
        MethodModel(n='目标y', f='ey', p='请输入ey坐标', d=True)])
    def a_drag(self, sx, sy, ex, ey):
        """坐标拖动"""
        self.base_data.android.drag(sx, sy, ex, ey, 0.5)

    @sync_method_callback('android', '页面操作', 7)
    def a_open_quick_settings(self):
        """打开快速通知"""
        self.base_data.android.open_quick_settings()

    @sync_method_callback('android', '页面操作', 8, [
        MethodModel(n='文件名称', f='file_name', p='请输入截图文件名称', d=True)])
    def a_screenshot(self, file_name: str):
        """屏幕截图"""
        self.base_data.android.screenshot(filename=os.path.join(self.base_data.screenshot_path, file_name))

    @sync_method_callback('android', '页面操作', 9, [
        MethodModel(n='x坐标', f='x', p='请输入按下的x坐标', d=True),
        MethodModel(n='y坐标', f='y', p='请输入按下的x坐标', d=True),
        MethodModel(n='长按时间', f='time_', p='请输入长按时间', d=True)])
    def a_long_click(self, x, y, time_):
        """长按屏幕N秒"""
        self.base_data.android.long_click(x, y, time_)

    @sync_method_callback('android', '页面操作', 10)
    def a_set_orientation_natural(self):
        """设置为natural"""
        self.base_data.android.set_orientation("natural")

    @sync_method_callback('android', '页面操作', 11)
    def a_set_orientation_left(self):
        """设置为natural"""
        self.base_data.android.set_orientation("left")

    @sync_method_callback('android', '页面操作', 12)
    def a_set_orientation_right(self):
        """设置为right"""
        self.base_data.android.set_orientation("right")

    @sync_method_callback('android', '页面操作', 13)
    def a_set_orientation_upsidedown(self):
        """设置为upsidedown"""
        self.base_data.android.set_orientation("upsidedown")

    @sync_method_callback('android', '页面操作', 14)
    def a_freeze_rotation(self):
        """冻结旋转"""
        self.base_data.android.freeze_rotation()

    @sync_method_callback('android', '页面操作', 15)
    def a_freeze_rotation_false(self):
        """取消冻结旋转"""
        self.base_data.android.freeze_rotation(False)

    @sync_method_callback('android', '页面操作', 16)
    def a_dump_hierarchy(self):
        """获取转储的内容"""
        return self.base_data.android.dump_hierarchy()

    @sync_method_callback('android', '页面操作', 17)
    def a_open_notification(self):
        """打开通知"""
        return self.base_data.android.dump_hierarchy()



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-04-12 15:53
# @Author : 毛鹏
import asyncio
import os
import random
import traceback
from typing import Optional

from playwright._impl._errors import TargetClosedError, Error

from mangotools.assertion import MangoAssertion
from mangotools.decorator import async_retry
from mangotools.enums import StatusEnum
from ..enums import ElementOperationEnum, DriveTypeEnum
from ..exceptions import MangoAutomationError
from ..exceptions.error_msg import *
from ..models import ElementResultModel, ElementModel, ElementListResultModel
from ..uidrive.android import AndroidDriver
from ..uidrive.web.async_web import AsyncWebDevice, AsyncWebAssertion


class AsyncElement(AsyncWebDevice, AndroidDriver):

    def __init__(self, base_data, drive_type: int, ):
        super().__init__(base_data)
        self.drive_type = drive_type
        self.element_model: Optional[ElementModel | None] = None
        self.element_result_model: Optional[ElementResultModel | None] = None
        self.element_list_model: Optional[list[ElementModel] | None] = None
        self.test_data = self.base_data.test_data

    async def open_device(self, is_open: bool = False):
        if self.drive_type == DriveTypeEnum.WEB.value:
            await self.open_url(is_open)
        elif self.drive_type == DriveTypeEnum.ANDROID.value:
            self.open_app()
        elif self.drive_type == DriveTypeEnum.DESKTOP.value:
            pass
        else:
            self.base_data.log.debug(f'不存在这个类型，如果是非管理员看到这种提示，请联系管理员')
            raise Exception('不存在的设备类型')

    async def element_main(self,
                           element_model: ElementModel,
                           element_list_model: list[ElementModel] | None = None) -> ElementResultModel:
        self.element_model = element_model
        self.element_list_model = element_list_model
        self.element_result_model = ElementResultModel(
            id=self.element_model.id,
            name=self.element_model.name,
            sleep=self.element_model.sleep,

            type=self.element_model.type.value,
            ope_key=self.element_model.ope_key,
            sql_execute=self.element_model.sql_execute,
            custom=self.element_model.custom,

            status=StatusEnum.FAIL.value,
        )
        try:
            await self.init_element()
            await self.__main()
            if self.element_model.sleep:
                await asyncio.sleep(self.element_model.sleep)
            self.element_result_model.status = StatusEnum.SUCCESS.value
            self.element_result_model.error_message = None
        except MangoAutomationError as error:
            self.base_data.log.debug(f'操作元素异常-1，类型：{type(error)}，失败详情：{error}')
            await self.__error(error.msg)
        except TargetClosedError as error:
            self.base_data.setup()
            self.base_data.log.debug(
                f'操作元素异常-2，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
            await self.__error(ERROR_MSG_0010[1], False)
        except Error as error:
            self.base_data.log.error(
                f'操作元素异常-3，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
            await self.__error(f'未知错误失败，请检查测试数据，如果需要明确的提示请联系管理员，提示：{error.message}')
        except Exception as error:
            error_msg = f'未知错误失败，请检查测试数据，如果需要明确的提示请联系管理员，提示：{error.args}'
            if hasattr(error, 'msg'):
                error_msg = error.msg
            self.base_data.log.error(
                f'操作元素异常-4，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
            await self.__error(error_msg)
        return self.element_result_model

    @async_retry()
    async def __main(self):
        self.base_data.verify_equipment(self.drive_type)
        if self.element_model.type == ElementOperationEnum.OPE:
            await self.__ope()
        elif self.element_model.type == ElementOperationEnum.ASS:
            await self.__ass()
        elif self.element_model.type == ElementOperationEnum.SQL:
            await self.__sql()
        elif self.element_model.type == ElementOperationEnum.CUSTOM:
            await self.__custom()
        elif self.element_model.type == ElementOperationEnum.CONDITION:
            await self.__condition()
        elif self.element_model.type == ElementOperationEnum.PYTHON_CODE:
            await self.__python_code()
        else:
            raise MangoAutomationError(*ERROR_MSG_0015)

    async def init_element(self):
        try:
            for i in self.element_model.elements:
                i.loc = self.base_data.test_data.replace(i.loc)
                i.sub = self.base_data.test_data.replace(i.sub)
            self.element_model.sleep = self.base_data.test_data.replace(self.element_model.sleep)
            self.element_result_model.sleep = self.element_model.sleep

        except MangoAutomationError as error:
            self.base_data.log.error(f'操作元素解析数据失败，类型：{type(error)}, 详情：{error}')
            raise MangoAutomationError(error.code, error.msg)

    async def __ope(self):
        method_name = getattr(self.element_model, 'ope_key', None)
        if not method_name:
            self.base_data.log.debug('操作失败-1，ope_key 不存在或为空')
            raise MangoAutomationError(*ERROR_MSG_0048)
        if not hasattr(self, method_name):
            self.base_data.log.debug(f'操作失败-2，方法不存在: {method_name}')
            raise MangoAutomationError(*ERROR_MSG_0048)
        if not callable(getattr(self, method_name)):
            self.base_data.log.debug(f'操作失败-3，属性不可调用: {method_name}')
            raise MangoAutomationError(*ERROR_MSG_0048)
        if self.element_model.ope_value is None:
            raise MangoAutomationError(*ERROR_MSG_0054)
        await self.__ope_value()
        if self.drive_type == DriveTypeEnum.WEB.value:
            await self.web_action_element(
                self.element_model.name,
                self.element_model.ope_key,
                {i.f: i.v for i in self.element_model.ope_value}
            )
        elif self.drive_type == DriveTypeEnum.ANDROID.value:
            self.a_action_element(
                self.element_model.name,
                self.element_model.ope_key,
                {i.f: i.v for i in self.element_model.ope_value}
            )
        else:
            pass
        self.element_result_model.ope_value = self.element_model.ope_value
        for i in self.element_result_model.ope_value:
            if not isinstance(i.v, str):
                i.v = str(i.v)

    async def __ass(self, _ope_value: dict | None = None):
        if self.element_model.ope_value is None:
            raise MangoAutomationError(*ERROR_MSG_0054)
        await self.__ope_value(True)
        ope_value = {i.f: i.v for i in self.element_model.ope_value}
        if _ope_value is not None:
            ope_value.update(_ope_value)
        try:
            if self.drive_type == DriveTypeEnum.WEB.value:
                self.element_result_model.ass_msg = await self.web_assertion_element(
                    self.element_model.name,
                    self.element_model.ope_key,
                    ope_value
                )
            elif self.drive_type == DriveTypeEnum.ANDROID.value:
                self.element_result_model.ass_msg = self.a_assertion_element(
                    self.element_model.name,
                    self.element_model.ope_key,
                    ope_value
                )
            else:
                pass
        except MangoAutomationError as error:
            self.element_result_model.ass_msg = error.msg
            raise error
        self.element_result_model.ope_value = self.element_model.ope_value
        for i in self.element_result_model.ope_value:
            if not isinstance(i.v, str):
                i.v = str(i.v)

    async def __sql(self):
        async def run(sql, key_list):
            result_list: list[dict] = self.base_data.mysql_connect.condition_execute(sql)
            self.base_data.log.debug(f'sql参数->key:{sql}，value:{result_list}')
            if not isinstance(result_list, list) and not len(result_list) > 0:
                raise MangoAutomationError(*ERROR_MSG_0036, value=(self.element_model.sql_execute,))
            self.base_data.test_data.set_sql_cache(key_list, result_list[0])

        if self.base_data.mysql_connect:
            for i in self.element_model.sql_execute:
                await run(i.get('sql'), i.get('key_list'))

    async def __custom(self):
        for i in self.element_model.custom:
            self.base_data.test_data.set_cache(i.get('key'), self.base_data.test_data.replace(i.get('value')))

    async def __condition(self):
        error_list = []
        for i in self.element_list_model:
            try:
                condition_value = self.base_data.test_data.replace(i.condition_value)
                self.base_data.log.debug(f'执行条件判断数据：{self.element_model.id}，{condition_value}')
                await self.__ass(condition_value)
                self.element_result_model.next_node_id = i.id
                return
            except Exception as error:
                self.base_data.log.debug(f'节点判断中-错误：{error}，明细：{traceback.print_exc()}')
                error_list.append(error)
        raise error_list[0]

    async def __python_code(self):
        self.base_data.log.debug(f'执行python函数：{self.element_model.func}')
        global_namespace = {}
        exec(self.element_model.func, global_namespace)
        global_namespace['func'](self)

    async def __ope_value(self, is_ass: bool = False):
        try:
            ope_key = 'actual' if is_ass else 'locating'
            for i in self.element_model.ope_value:
                if i.f == ope_key and self.element_model.elements:
                    random_element = random.randint(0, len(self.element_model.elements) - 1)
                    find_params = {
                        'name': self.element_model.name,
                        '_type': self.element_model.type,
                        'exp': self.element_model.elements[random_element].exp,
                        'loc': self.element_model.elements[random_element].loc,
                        'sub': self.element_model.elements[random_element].sub
                    }
                    if self.drive_type == DriveTypeEnum.WEB.value:
                        loc, ele_quantity, element_text = await self.web_find_ele(
                            **find_params, is_iframe=self.element_model.elements[random_element].is_iframe)
                    elif self.drive_type == DriveTypeEnum.ANDROID.value:
                        loc, ele_quantity, element_text = self.a_find_ele(**find_params)
                    else:
                        loc, ele_quantity, element_text = None, 0, None
                    new_element = ElementListResultModel(
                        exp=self.element_model.elements[random_element].exp,
                        loc=self.element_model.elements[random_element].loc,
                        sub=self.element_model.elements[random_element].sub,
                        ele_quantity=ele_quantity,
                        element_text=element_text,
                        is_iframe=self.element_model.elements[random_element].is_iframe
                    )

                    element_exists = any(
                        existing.exp == new_element.exp and
                        existing.loc == new_element.loc and
                        existing.sub == new_element.sub and
                        existing.is_iframe == new_element.is_iframe
                        for existing in self.element_result_model.elements
                    )

                    if not element_exists:
                        self.element_result_model.elements.append(new_element)
                    if is_ass:
                        if callable(getattr(AsyncWebAssertion, self.element_model.ope_key, None)):
                            i.v = loc
                        elif callable(getattr(MangoAssertion(), self.element_model.ope_key, None)):
                            i.v = element_text
                    else:
                        i.v = loc
                i.v = self.base_data.test_data.replace(i.v)

        except AttributeError as error:
            self.base_data.log.debug(
                f'获取操作值失败-1，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
            raise MangoAutomationError(*ERROR_MSG_0027)

    async def __error(self, msg: str, is_screenshot: bool = True):
        try:
            self.element_result_model.status = StatusEnum.FAIL.value
            self.element_result_model.error_message = msg
            self.base_data.log.debug(
                f"""
                元素操作失败----->
                元 素 对 象：{self.element_model.model_dump() if self.element_model else self.element_model}
                元素测试结果：{
                self.element_result_model.model_dump() if self.element_result_model else self.element_result_model}
                """
            )
            if is_screenshot:
                file_name = f'失败截图-{self.element_model.name}{self.base_data.test_data.get_time_for_min()}.jpg'
                file_path = os.path.join(self.base_data.screenshot_path, file_name)
                await self.__error_screenshot(file_path)
                self.element_result_model.picture_path = file_path
                self.element_result_model.picture_name = file_name
        except MangoAutomationError as error:
            self.element_result_model.error_message += f'执行过程中发生失败，准备截图时截图失败，失败原因：{error.msg}'
        except Exception as error:
            self.base_data.log.error(
                f'截图失败未知异常-1，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')

    async def __error_screenshot(self, file_path):
        if self.drive_type == DriveTypeEnum.WEB.value:
            await self.w_screenshot(file_path)
        elif self.drive_type == DriveTypeEnum.ANDROID.value:
            self.a_screenshot(file_path)
        else:
            pass



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description:# @Time   : 2023-09-09 23:17
# @Author : 毛鹏
import sys

from ..models._ui_model import ElementModel, ElementResultModel, ElementListResultModel


__all__ = [
    'ElementModel',
    'ElementResultModel',
    'ElementListResultModel'
]



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-04-10 21:37
# @Author : 毛鹏
class Meta(type):

    def __new__(cls, name, bases, attrs, **kwargs):
        methods = kwargs.pop('methods', {})
        for method_name, method_func in methods.items():
            attrs[method_name] = method_func
        return super().__new__(cls, name, bases, attrs)



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: # @Time   : 2023-07-15 11:57
# @Author : 毛鹏
import sys

from ..uidrive._async_element import AsyncElement
from ..uidrive._base_data import BaseData
from ..uidrive._driver_object import DriverObject
from ..uidrive._sync_element import SyncElement


__all__ = [
    'AsyncElement',
    'BaseData',
    'DriverObject',
    'SyncElement',
]



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description:
# @Time   : 2024-04-24 10:43
# @Author : 毛鹏

from typing import Optional

from ..uidrive.android._new_android import NewAndroid
from ..uidrive.pc.new_windows import NewWindows
from ..uidrive.web.async_web._new_browser import AsyncWebNewBrowser
from ..uidrive.web.sync_web._new_browser import SyncWebNewBrowser


class DriverObject:

    def __init__(self, log, is_async=False):
        self.log = log
        self.is_async = is_async
        self.web: Optional[AsyncWebNewBrowser | SyncWebNewBrowser] = None
        self.android: Optional[NewAndroid] = None
        self.windows: Optional[NewWindows] = None

    def set_web(self,
                web_type: int,
                web_path: str | None = None,
                web_max=False,
                web_headers=False,
                web_recording=False,
                web_h5=None,
                is_header_intercept=False,
                web_is_default=False,
                videos_path=None
                ):
        if self.is_async:
            self.web = AsyncWebNewBrowser(
                web_type,
                web_path,
                web_max,
                web_headers,
                web_recording,
                web_h5,
                is_header_intercept,
                web_is_default,
                videos_path,
                log=self.log,
            )
        else:
            self.web = SyncWebNewBrowser(
                web_type,
                web_path,
                web_max,
                web_headers,
                web_recording,
                web_h5,
                is_header_intercept,
                web_is_default,
                videos_path,
                log=self.log,
            )

    def set_android(self, and_equipment: str):
        self.android = NewAndroid(and_equipment)

    def set_windows(self, win_path: str, win_title: str):
        self.windows = NewWindows(win_path, win_title)



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: # @Time   : 2023-04-25 22:33
# @Author : 毛鹏
import time
from playwright.sync_api import Locator

from ....tools import Meta
from ....uidrive._base_data import BaseData
from mangotools.decorator import sync_method_callback
from mangotools.models import MethodModel


class SyncWebPage(metaclass=Meta):
    """页面操作"""

    def __init__(self, base_data: BaseData):
        self.base_data = base_data

    @sync_method_callback('web', '页面操作', 0, [
        MethodModel(n='页签下标', f='individual', p='请输入页签下标，从1开始数', d=True)])
    def w_switch_tabs(self, individual: int):
        """切换页签"""
        pages = self.base_data.context.pages
        pages[int(individual) + 1].bring_to_front()
        self.base_data.page = pages[int(individual) + 1]
        time.sleep(1)

    @sync_method_callback('web', '页面操作', 1)
    def w_close_current_tab(self):
        """关闭当前页签"""
        time.sleep(2)
        pages = self.base_data.context.pages
        pages[-1].close()
        self.base_data.page = pages[0]

    @sync_method_callback('web', '页面操作', 2, [MethodModel(f='locating')])
    def w_open_new_tab_and_switch(self, locating: Locator):
        """点击并打开新页签"""
        locating.click()
        time.sleep(2)
        pages = self.base_data.context.pages
        new_page = pages[-1]
        new_page.bring_to_front()
        self.base_data.page = new_page
        time.sleep(1)

    @sync_method_callback('web', '页面操作', 3)
    def w_refresh(self):
        """刷新页面"""
        self.base_data.page.reload()

    @sync_method_callback('web', '页面操作', 4)
    def w_go_back(self):
        """返回上一页"""
        self.base_data.page.go_back()

    @sync_method_callback('web', '页面操作', 5)
    def w_go_forward(self):
        """前进到下一页"""
        self.base_data.page.go_forward()



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-04-12 17:31
# @Author : 毛鹏
import asyncio
import unittest

from mangoautomation.models import ElementModel
from mangoautomation.uidrive import AsyncElement, BaseData, DriverObject, SyncElement
from mangotools.data_processor import DataProcessor
from mangotools.log_collector import set_log

log = set_log('D:\GitCode\mango_automation\logs')
test_data = DataProcessor()
element_model_1 = ElementModel(**{
    "id": 9,
    "type": 0,
    "name": "搜索结果",
    "loc": "get_by_role(\"link\", name=\"芒果自动化测试平台: 芒果测试平台是UI和API的自动化测试\")",
    "exp": 2,
    "sleep": None,
    "sub": None,
    "is_iframe": 0,
    "ope_key": "w_open_new_tab_and_switch",
    "ope_value": [
        {
            "f": "locating",
            "p": None,
            "d": False,
            "v": ""
        }
    ],
    "key_list": None,
    "sql": None,
    "key": None,
    "value": None
})
element_model_2 = ElementModel(**{
    "id": 10, "type": 2, "name": "结果内容", "loc": "get_by_role(\"heading\", name=\"芒果测试平台是集UI，API\")",
    "exp": 0,
    "sleep": None, "sub": None, "is_iframe": 0, "ope_key": "w_get_text",
    "ope_value": [{"f": "locating", "p": None, "d": False, "v": ""},
                  {"f": "set_cache_key", "p": None, "d": True, "v": "结果内容"}]
})


class TestUi(unittest.IsolatedAsyncioTestCase):
    async def test_a(self):
        driver_object = DriverObject(True)
        driver_object.set_web(0, r"C:\Program Files\Google\Chrome\Application\chrome.exe")
        base_data = BaseData(test_data, log)
        base_data.log = log
        base_data.url = 'https://www.baidu.com/'

        base_data.context, base_data.page = await driver_object.web.new_web_page()
        element = AsyncElement(base_data, element_model_1, 0)
        await element.open_url()
        await asyncio.sleep(5)
        await element.element_main()
        print(element.element_result_model.model_dump())
        element = AsyncElement(base_data, element_model_2, 0)
        await element.element_main()
        print(element.element_result_model.model_dump())


class TestUi2(unittest.TestCase):

    def test_s(self):
        driver_object = DriverObject()
        driver_object.set_web(0, r"C:\Program Files\Google\Chrome\Application\chrome.exe")
        base_data = BaseData(test_data, log)
        base_data.url = 'https://www.baidu.com/'
        base_data.context, base_data.page = driver_object.web.new_web_page()
        element = SyncElement(base_data, element_model_1, 0)
        element.open_url()
        element.element_main()



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2024-05-23 15:05
# @Author : 毛鹏

from typing import Optional

import uiautomator2 as u2
from adbutils import AdbTimeout
from uiautomator2 import ConnectError

from ...exceptions import MangoAutomationError
from ...exceptions.error_msg import ERROR_MSG_0042, ERROR_MSG_0045, ERROR_MSG_0040

"""
python -m uiautomator2 init
python -m weditor

"""


class NewAndroid:

    def __init__(self, and_equipment):
        self.and_equipment = and_equipment
        self.info: Optional[dict | None] = None
        self.example_dict = []

    def new_android(self):
        if self.and_equipment is None:
            raise MangoAutomationError(*ERROR_MSG_0042)
        try:

            android = u2.connect(self.and_equipment)
            self.info = android.info
            # msg = f"设备启动成功！产品名称：{self.info.get('productName')}"
            self.example_dict.append({
                'config': self.and_equipment,
                'info': self.info,
                'android': android
            })
        except ConnectError:
            raise MangoAutomationError(*ERROR_MSG_0040, value=(self.and_equipment,))
        except RuntimeError:
            raise MangoAutomationError(*ERROR_MSG_0045, value=(self.and_equipment,))
        except (AdbTimeout, TimeoutError):
            raise MangoAutomationError(*ERROR_MSG_0040, value=(self.and_equipment,))
        else:
            android.implicitly_wait(10)
            return android

    def close_android(self):
        pass



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: # @Time   : 2023-04-25 22:33
# @Author : 毛鹏
import json
import os.path
import traceback
from urllib.parse import urlparse

import time
from playwright._impl._errors import TimeoutError, Error, TargetClosedError

from mangotools.decorator import sync_method_callback
from mangotools.models import MethodModel
from ....exceptions import MangoAutomationError
from ....exceptions.error_msg import ERROR_MSG_0049, ERROR_MSG_0013, ERROR_MSG_0058, ERROR_MSG_0059, ERROR_MSG_0010
from ....tools import Meta
from ....uidrive._base_data import BaseData


class SyncWebBrowser(metaclass=Meta):
    """浏览器操作"""

    def __init__(self, base_data: BaseData):
        self.base_data = base_data

    @sync_method_callback('web', '浏览器操作', 1, [
        MethodModel(n='等待时间', f='_time', p='请输入等待时间', d=True)])
    def w_wait_for_timeout(self, _time: int):
        """强制等待"""
        time.sleep(int(_time))

    @sync_method_callback('web', '浏览器操作', 1, [
        MethodModel(n='url地址', f='url', p='请输入URL', d=True)])
    def w_goto(self, url: str):
        """打开URL"""
        try:
            result = urlparse(url)
            if not all([result.scheme, result.netloc]):
                raise MangoAutomationError(*ERROR_MSG_0049)
            self.base_data.page.goto(url, timeout=60000)
            time.sleep(2)
        except TimeoutError as error:
            self.base_data.log.debug(f'打开URL失败-2，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0013, value=(url,))
        except TargetClosedError as error:
            self.base_data.setup()
            self.base_data.log.debug(f'打开URL失败-2，类型：{type(error)}，失败详情：{error}')
            raise MangoAutomationError(*ERROR_MSG_0010, value=(url,))
        except Error as error:
            self.base_data.log.debug(
                f'打开URL失败-3，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
            raise MangoAutomationError(*ERROR_MSG_0058, value=(url,))

    @sync_method_callback('web', '浏览器操作', 2, [
        MethodModel(n='保存路径', f='path', p='请输入截图保存路径', d=True)])
    def w_screenshot(self, path: str):
        """整个页面截图"""
        try:
            self.base_data.page.screenshot(path=path, full_page=True, timeout=10000)
        except (TargetClosedError, TimeoutError) as error:
            self.base_data.log.debug(
                f'截图出现异常失败-1，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
            self.base_data.setup()
            raise MangoAutomationError(*ERROR_MSG_0010)
        except AttributeError as error:
            self.base_data.log.debug(
                f'截图出现异常失败-2，类型：{type(error)}，失败详情：{error}，失败明细：{traceback.format_exc()}')
            self.base_data.setup()
            raise MangoAutomationError(*ERROR_MSG_0010)

    @sync_method_callback('web', '浏览器操作', 3)
    def w_alert(self):
        """设置弹窗不予处理"""
        self.base_data.page.on("dialog", lambda dialog: dialog.accept())

    @sync_method_callback('web', '浏览器操作', 4)
    def w_get_cookie(self):
        """获取cookie"""
        with open(os.path.join(self.base_data.download_path, 'storage_state.json'), 'w') as file:
            file.write(json.dumps(self.base_data.context.storage_state()))

    @sync_method_callback('web', '浏览器操作', 5, [
        MethodModel(n='获取cookie的值', f='storage_state', p='请输入获取cookie方法中获取的内容', d=True)])
    def w_set_cookie(self, storage_state: str):
        """设置cookie"""
        if isinstance(storage_state, str):
            storage_state = json.loads(storage_state)
        else:
            raise MangoAutomationError(*ERROR_MSG_0059)
        self.base_data.context.add_cookies(storage_state['cookies'])
        for storage in storage_state['origins']:
            local_storage = storage.get('localStorage', [])
            session_storage = storage.get('sessionStorage', [])
            for item in local_storage:
                self.base_data.context.add_init_script(
                    f"window.localStorage.setItem('{item['name']}', '{item['value']}');")
            for item in session_storage:
                self.base_data.context.add_init_script(
                    f"window.sessionStorage.setItem('{item['name']}', '{item['value']}');")
        self.base_data.page.reload()

    @sync_method_callback('web', '浏览器操作', 6)
    def w_clear_cookies(self):
        """清除所有cookie"""
        self.base_data.context.clear_cookies()

    @sync_method_callback('web', '浏览器操作', 7)
    def w_clear_storage(self):
        """清除本地存储和会话存储"""
        self.base_data.page.evaluate("() => localStorage.clear()")
        self.base_data.page.evaluate("() => sessionStorage.clear()")



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2025-04-15 11:54
# @Author : 毛鹏
import sys

from ..tools._mate import Meta
from ..tools._uiautodev import start_uiautodev, stop_uiautodev


__all__ = ['Meta', 'start_uiautodev', 'stop_uiautodev']



# -*- coding: utf-8 -*-
# @Project: 芒果测试平台
# @Description: 
# @Time   : 2024-11-30 13:12
# @Author : 毛鹏
import subprocess
from typing import Optional

import sys
import time

if not sys.platform.startswith('win32'):
    print("警告: uiautomation 仅支持 Windows，当前环境已自动跳过")


    class Auto:
        pass


    auto = Auto()


    class Control:
        pass
else:

    from uiautomation import Control, WindowControl
    import uiautomation


class NewWindows:

    def __init__(self, win_path: str, win_title: str, timeout=10):
        self.win_path = win_path
        self.win_title = win_title
        self.timeout = timeout
        self.window: Optional[None | WindowControl] = None

    def new_windows(self, retry=3):
        subprocess.Popen(self.win_path)
        for _ in range(retry):
            self.window = uiautomation.WindowControl(
                searchDepth=1,
                Name=self.win_title,
                Timeout=self.timeout * 1000
            )
            if self.window.Exists():
                return self.window
            time.sleep(1)

    def find_control(self,
                     parent=None,
                     control_type: str = "Control",
                     name: str = None,
                     automation_id: str = None,
                     **kwargs) -> Control:
        """
        查找控件（支持多条件组合）
        :param parent: 父控件（默认从窗口开始查找）
        :param control_type: 控件类型（如 Button、Edit、ListItem）
        :param name: 控件名称（Name 属性）
        :param automation_id: AutomationId 属性
        :return: Control 对象
        """
        search_params = {
            "searchDepth": 2,
            "Timeout": self.timeout * 1000,
            **kwargs
        }
        if name:
            search_params["Name"] = name
        if automation_id:
            search_params["AutomationId"] = automation_id

        parent = parent or self.window
        control = getattr(uiautomation, f"{control_type}Control")(parent=parent, **search_params)
        if not control.Exists():
            raise Exception(f"未找到控件: {control_type}/{name}/{automation_id}")
        return control
